<?php
/**
 * class handler
 * This class handles all the request to the script. It does:
 * 		- Makes canonical redirection
 * 		- Sets the propper theme template file
 * 		- Fetch all the access to Chevereto
 *
 * @version		1.0
 * @author		Rodolfo Berríos <http://rodolfoberrios.com/>
 * @url			<http://godlikestudio.com>
 * @package		Chevereto <http://chevereto.com>
 *
 * @copyright	Rodolfo Berríos <inbox@rodolfoberrios.com>
 *
 */

class Handler {

	private $root_url;
	private $base_url;
	private $path_images;
	private $path_thumbs;
	private $path_themes;
	
	private $request_uri;
	private $script_name;
	private $valid_request;
	private $canonical_request;
	
	private $template;
	private $pages;
	
	private $handled_request;
	private $request_array;
	
	public static $doctitle;
	public static $img_filename;
	public static $img_path;
	public static $thumb_path;
	public static $img_url;
	public static $thumb_url;
	public static $img_viewer;
	
	// Let's construct a valid request!
	function __construct($server_request)
	{
		// Parse the definitions to this object.. This nos not necessary but in case of changes...
		$this->root_url = __RELATIVE_ROOT__;
		$this->base_url = __BASE_URL__;
		$this->path_images = rtrim(__PATH_IMAGES__,'/').'/';
		$this->path_thumbs = rtrim(__PATH_THUMBS__,'/').'/';
		$this->path_theme = __PATH_THEME__;
		
		// Parse the params
		$this->request_uri = $server_request['REQUEST_URI'];
		$this->script_name = $server_request['SCRIPT_NAME'];
		$this->valid_request = $this->sanitize_path($this->request_uri);

		// Build the canonical request
		$this->canonical_request = '/'.$this->valid_request;
		
		// If the request is invalid we make a 301 redirection to the canonical url.
		if($this->root_url!==$this->request_uri and $this->canonical_request!==$this->request_uri) {
			$this->redirect($this->base_redirection($this->canonical_request), 301);
			die();
		}
		
		// It's a valid request on index.php so we do the thing...
		if($this->is_index()) $this->proccess_request();

	}
	
	
	private function proccess_request()
	{
		global $lang;
		$this->template = 404; // Default template
		$this->pages = $this->get_pages(); // get theme pages
		$this->handled_request = ($this->root_url=='/') ? $this->valid_request : str_ireplace($this->root_url, '', $this->add_trailing_slashes($this->request_uri));
		$this->request_array = explode('/', rtrim($this->handled_request, '/'));
		
		$get_v = (isset($_GET['v'])) ? $_GET['v'] : '';
		
		// Switch according the request
		switch($this->request_array[0]) {

			case '':
			case 'index.php':
				$this->template = 'index';
			break;
			
			case 'error-javascript':
				chevereto_die(array($lang['critical_js_step_1'], $lang['critical_js_step_2']),'JavaScript', array($lang['critical_js']));
			break;
			
			
			case '?uploaded':
				session_id($_COOKIE['PHPSESSID']);
				session_start();
				if(count($_SESSION['ImagesUp'])>0) {
					//unset($_SESSION['ImagesUp']);
					$img_filename = $_SESSION['ImagesUp'][$this->getArrayFirstIndex($_SESSION['ImagesUp'])]['img_name'];
					$this->template = 'uploaded';
					self::$doctitle = $lang['doctitle_upload_complete'];
				} else {
					$this->redirect(__BASE_URL__, 400);
					die();
				}
			break;
			
			case '?v='.$get_v: // View request
				$img_filename = $_GET['v'];
				if ($this->is_valid_image($this->path_images.$img_filename)) {
					$this->template = 'view';
					self::$doctitle = $lang['doctitle_viewing_image'].' '.$img_filename;				
				}
			break;
			
			case '?chevereto':
				$this->template = 'bool';
			break;
			
			default: // Pages request
				require_once($this->path_theme.'pages/pages_config.php'); // We load the special pages config
				if(in_array($this->request_array[0].'.php', $this->pages) and $this->request_array[1]=='' and $pages_config[$this->request_array[0]]['live']) {
					$this->template = 'pages/'.$this->request_array[0];
					self::$doctitle = $pages_config[$this->request_array[0]]['title'];
				}
			break;
		}
		
		if($this->template=='uploaded' or $this->template=='view') {
			$img_path = $this->path_images.$img_filename;
			$thumb_path = $this->path_thumbs.$img_filename;
			self::$img_filename = $img_filename;
			self::$img_path = $img_path;
			self::$thumb_path = $thumb_path;
			self::$img_url = absolute_to_url($img_path);
			self::$thumb_url = absolute_to_url($thumb_path);
			self::$img_viewer = __BASE_URL__.'?v='.$img_filename;
		}
		
		if($this->template == 404) {
			$this->status_header(404);
			self::$doctitle = $lang['txt_404_title'];
		} else {
			$this->status_header(200);
		}
		
		// We load the template
		if($this->template == 'bool'){
			exit(json_encode(true));
		} else {
			$this->load_template();
		}
	}

	
	/**
	 * sanitize_path
	 * Sanitizes ///ugly//path/// to clean/path
	 */
	private function sanitize_path($path)
	{
		return implode('/', array_filter(explode('/', $path)));
	}


	/**
	 * add_trailing_slashes
	 * Adds allways trailing slash to string.
	 * @Inspired from WordPress
	 */	
	private function add_trailing_slashes($string)
	{
		$string = '/'.ltrim($string, '/');
		return rtrim($string, '/') . '/';
	}
	
	private function getArrayFirstIndex($arr)
	{
      foreach ($arr as $key => $value)
      return $key;
	}



	/**
	 * redirect
	 * Redirects to another URL
	 * @Inspired from WordPress
	 */	
	private function redirect($to, $status=301)
	{
		$to = preg_replace('|[^a-z0-9-~+_.?#=&;,/:%!]|i', '', $to);
		if(php_sapi_name() != 'cgi-fcgi') $this->status_header($status);
		header("Location: $to");
	}


	/**
	 * base_redirection
	 * Returns the correct url to handle redirections replacing the extra /path/ from request
	 */
	private function base_redirection($request)
	{
		return str_replace($this->root_url, '/', $this->base_url).ltrim($request, '/');
	}


	/**
	 * status_header
	 * Set HTTP status header from status code
	 * @Inspired from WordPress
	 */
	private function status_header($code)
	{
		$desc = $this->get_status_header_desc($code);
		if (empty($desc)) return false;
		$protocol = $_SERVER["SERVER_PROTOCOL"];
		if ('HTTP/1.1' != $protocol && 'HTTP/1.0' != $protocol) $protocol = 'HTTP/1.0';
		$status_header = "$protocol $code $desc";
		return @header($status_header, true, $code);
	}

	
	/**
	 * get_status_header_desc
	 * Gets header desc according to it's code
	 * @Inspired from WordPress
	 */
	private function get_status_header_desc($code)
	{
		$codes_to_desc = array(
				100 => 'Continue',
				101 => 'Switching Protocols',
				102 => 'Processing',
	
				200 => 'OK',
				201 => 'Created',
				202 => 'Accepted',
				203 => 'Non-Authoritative Information',
				204 => 'No Content',
				205 => 'Reset Content',
				206 => 'Partial Content',
				207 => 'Multi-Status',
				226 => 'IM Used',
	
				300 => 'Multiple Choices',
				301 => 'Moved Permanently',
				302 => 'Found',
				303 => 'See Other',
				304 => 'Not Modified',
				305 => 'Use Proxy',
				306 => 'Reserved',
				307 => 'Temporary Redirect',
	
				400 => 'Bad Request',
				401 => 'Unauthorized',
				402 => 'Payment Required',
				403 => 'Forbidden',
				404 => 'Not Found',
				405 => 'Method Not Allowed',
				406 => 'Not Acceptable',
				407 => 'Proxy Authentication Required',
				408 => 'Request Timeout',
				409 => 'Conflict',
				410 => 'Gone',
				411 => 'Length Required',
				412 => 'Precondition Failed',
				413 => 'Request Entity Too Large',
				414 => 'Request-URI Too Long',
				415 => 'Unsupported Media Type',
				416 => 'Requested Range Not Satisfiable',
				417 => 'Expectation Failed',
				422 => 'Unprocessable Entity',
				423 => 'Locked',
				424 => 'Failed Dependency',
				426 => 'Upgrade Required',
	
				500 => 'Internal Server Error',
				501 => 'Not Implemented',
				502 => 'Bad Gateway',
				503 => 'Service Unavailable',
				504 => 'Gateway Timeout',
				505 => 'HTTP Version Not Supported',
				506 => 'Variant Also Negotiates',
				507 => 'Insufficient Storage',
				510 => 'Not Extended'
		);
		
		if (isset($codes_to_desc[$code])) {
			return $codes_to_desc[$code];
		} else {
			return '';
		}
	}
	
	
	/**
	 * is_index
	 * We look at the script_name to see if we are requesting index.php
	 */	
	private function is_index()
	{
		return preg_match('{/index.php$}', $this->script_name);
	}
	
	
	/**
	 * is_valid_image
	 * We look if the image exists and if is valid
	 */
	private function is_valid_image($image) {
		if(file_exists($image) and @getimagesize($image)) {
			return true;
		}
	}

	
	private function get_pages()
	{
		$pages = scandir(__PATH_THEME__.'pages/');
		return array_values(array_diff($pages, array('.', '..', 'pages_config.php', 'index.php')));
	}

	
	private function load_template()
	{
		if(file_exists($this->path_theme.'functions.php')) require_once($this->path_theme.'functions.php');
		require_once($this->path_theme.$this->template.'.php');	// Content
	}

	
	public function get_img_info($what)
	{
		$img = self::$img_path;
		$info = getimagesize($img);
		$stat = stat($img);
		switch($what) {
			case 'width':
				return $info[0];
			break;
			case 'height':
				return $info[1];
			break;
			case 'html':
				return $info[3];
			break;
			case 'weight':
				return bytes_to_kb($stat['size']);
			break;
		}
	}

}

/**
 * Procedural methods->functions
 * This functions are made to be used as procedural versions of the class handler methods.
 */
function get_doctitle() {
	return Handler::$doctitle;
}

function get_img_url() {
	return Handler::$img_url;
}

function get_img_relative_url() {
	return absolute_to_relative(Handler::$img_path);
}

function get_thumb_url() {
	return Handler::$thumb_url;
}

function get_img_filename() {
	return Handler::$img_filename;
}

function get_img_viewer() {
	return Handler::$img_viewer;
}

function get_img_width() {
	return Handler::get_img_info('width');
}

function get_img_height() {
	return Handler::get_img_info('height');
}

function get_img_html_size() {
	return Handler::get_img_info('html');
}

function get_img_weight() {
	return Handler::get_img_info('weight');
}

?>