<?php
/**
 * class.shorturl.php
 * This class uses the most popular url shortening services to short a long url
 *
 * @version		1.0
 * @author		Rodolfo Berríos <http://rodolfoberrios.com/>
 * @url			<http://godlikestudio.com>
 * @package		Chevereto <http://chevereto.com>
 *
 * @copyright	Rodolfo Berríos <inbox@rodolfoberrios.com>
 *
 */

class ShortURL {
	public $url;
	public $service;
	public $user;
    public $pass;
	
	public $custom_service_api;

	
    /**
     * get_ShortURL
     *
     * Get the processed URL.
     *
     * @access	public
     * @param	string
     * @return	string
     */	
	function get_ShortURL($short_url) {
		$this->url = $short_url;
		switch($this->service) {
			case 'tinyurl':
				return $this->tinyurl($this->url);
			break;
			case 'google':
				return $this->google($this->url);
			break;
			case 'isgd':
				return $this->isgd($this->url);
			break;
			case 'bitly':
				return $this->bitly($this->url);
			break;
			case 'custom':
				return $this->custom($this->url);
			break;
		}
	}
	
	/**
	 * fetch_url
	 *
	 * Gets the url content for a given URL using the cURL library
	 *
     * @access	private
     * @param	string
     * @return	string
	 */
	private function fetch_url($url) {
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_TIMEOUT, 120);
		$file_get_contents = curl_exec($ch);
		curl_close($ch);
		return $file_get_contents;
	}
	
	
    /**
     * tinyurl
     *
     * @access	private
     * @param	string
     * @return	string
     */
    private function tinyurl($url) {
        return $this->fetch_url('http://tinyurl.com/api-create.php?url='.urlencode($url));
    }
    
	
    /**
     * isgd
     *
     * @access	private
     * @param	string
     * @return	string
     */
    private function isgd($url) {
		return $this->fetch_url('http://is.gd/api.php?longurl='.urlencode($url));
    }


    /**
     * bitly
     *
     * @access	private
     * @param	string
     * @return	string
     */
    private function bitly($url) {
        return $this->fetch_url('http://api.bit.ly/v3/shorten?login='.$this->user.'&apiKey='.$this->pass.'&longUrl='.urlencode($url).'&format=txt');
    }


    /**
     * custom
     *
     * @access	private
     * @param	string
     * @return	string
     */
    private function custom($url) {
        return $this->fetch_url($this->custom_service_api.urlencode($url));
    }

	 
    /**
	 * Google functions thanks to Marcus Nunes - marcusnunes.com
	 *
     * @access	private
     * @param	string
     * @return	string
     */
    private function google($url)
	{	       
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, 'http://goo.gl/api/url');
        curl_setopt($curl, CURLOPT_POST, 1);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curl, CURLOPT_POSTFIELDS, 'user=toolbar@google.com&url='.urlencode($url).'&auth_token='.$this->googlToken($url));
        $return = curl_exec($curl);
        curl_close($curl);
        if($return) {
            $json = json_decode($return);
            return $json->short_url;
        }
    }
    
    //token code
	private function googlToken($b)
	{
		$i = $this->tke($b);
		$i = $i >> 2 & 1073741823;
		$i = $i >> 4 & 67108800 | $i & 63;
		$i = $i >> 4 & 4193280 | $i & 1023;
		$i = $i >> 4 & 245760 | $i & 16383;
		$j = "7";
		$h = $this->tkf($b);
		$k = ($i >> 2 & 15) << 4 | $h & 15;
		$k |= ($i >> 6 & 15) << 12 | ($h >> 8 & 15) << 8;
		$k |= ($i >> 10 & 15) << 20 | ($h >> 16 & 15) << 16;
		$k |= ($i >> 14 & 15) << 28 | ($h >> 24 & 15) << 24;
		$j .= $this->tkd($k);
		return $j;
	}

	private function tkc()
	{
		$l = 0;
		foreach(func_get_args() as $val){
			$val &= 4294967295;
			$val += $val > 2147483647 ? -4294967296 : ($val < -2147483647 ? 4294967296 : 0);
			$l   += $val;
			$l   += $l > 2147483647 ? -4294967296 : ($l < -2147483647 ? 4294967296 : 0);
		}
		return $l;
	}

	private function tkd($l)
	{
		$l = $l > 0 ? $l : $l + 4294967296;
		$m = "$l";
		$o = 0;
		$n = false;
		for($p = strlen($m) - 1; $p >= 0; --$p){
			$q = $m[$p];
			if($n){
				$q *= 2;
				$o += floor($q / 10) + $q % 10;
			} else {
				$o += $q;
			}
			$n = !$n;
		}
		$m = $o % 10;
		$o = 0;
		if($m != 0){
			$o = 10 - $m;
			if(strlen($l) % 2 == 1){
				if ($o % 2 == 1){
					$o += 9;
				}
				$o /= 2;
			}
		}
		return "$o$l";
	}

	private function tke($l)
	{
		$m = 5381;
		for($o = 0; $o < strlen($l); $o++){
			$m = $this->tkc($m << 5, $m, ord($l[$o]));
		}
		return $m;
	}

	private function tkf($l)
	{
		$m = 0;
		for($o = 0; $o < strlen($l); $o++){
			$m = $this->tkc(ord($l[$o]), $m << 6, $m << 16, -$m);
		}
		return $m;
	}

}
?>