<?php
/**
 * class.upload.php
 * This class is used to do image upload from URL or $_FILES
 * Also includes thumbnail generation and resize.
 *
 * @version		1.0
 * @author		Rodolfo Berríos <http://rodolfoberrios.com/>
 * @url			<http://godlikestudio.com>
 * @package		Chevereto <http://chevereto.com>
 *
 * @requires	- GD library <http://www.libgd.org/>
 *				- cURL <http://php.net/manual/en/book.curl.php>
 *				- class.imageconvert.php
 *				- chevereto functions.php
 *
 * @copyright	Rodolfo Berríos <inbox@rodolfoberrios.com>
 *
 */

class Upload {
	public $is_remote;
	public $img_upload_path;
	public $thumb_upload_path;
	
	public $resize_width;
	public $thumb_width;
	public $thumb_height;
	
	public $max_size;
	
	public $error;
	public $image_info;
	
	private $source;
	private $info;
	private $mime;
	private $extension;
	private $working;
	

	// Grab the source
	function __construct($source) {
		$this->source = $source;
	}

	
	/**
	 * process
	 * Does the thing
	 *
	 * @access	public
	 * @param	string
	 * @return	mixed
	 */
	public function process()
	{	
		if($this->valid_data()) {
			$this->extension = $this->get_true_extension($this->mime);		
			if($this->extension == 'bmp') {
				require_once('class.imageconvert.php');
				$this->ImageConvert = new ImageConvert($this->working, $this->extension, $this->img_upload_path.'temp_'.generateRandomString(256));
				unset($this->working); unset($this->extension);
				$this->working = $this->ImageConvert->out;
				$this->extension = 'png';
			}
			$image_uploaded_path = $this->randomFile($this->img_upload_path, $this->extension);
			
			// Prepare and formats the temp image
			$formated_temp = $this->working.'.'.$this->extension;
			rename($this->working, $formated_temp);
			unset($this->working); $this->working = $formated_temp;
			
			// Thumb
			$thumb_filename = str_replace($this->img_upload_path, $this->thumb_upload_path, $image_uploaded_path);
			$this->resize($this->working, $thumb_filename, $this->thumb_width, $this->thumb_height, true);
			
			// Resize?
			if(check_value($this->resize_width)) {
				$this->resize($this->working, $this->working, $this->resize_width);
			}
			
			if(!check_value($this->error)) {
				// Move the temp to the final path...
				$uploaded = rename($this->working, $image_uploaded_path);
				
				// Change the CHMOD of the file (for some php enviroments)
				@chmod($image_uploaded_path, 0644);
				@chmod($thumb_filename, 0644);
				
				if($uploaded) {
					list($width, $height, $type, $html) = $this->get_info($image_uploaded_path);
					$statinfo = stat($image_uploaded_path);
					$image_relative_path = absolute_to_relative($image_uploaded_path);
					$image_url = absolute_to_url($image_uploaded_path);
					$image_name = str_replace($this->img_upload_path, '', $image_uploaded_path);
									
					$this->image_info = array(
						'img_name'		=>	$image_name,
						'img_path'		=>	$image_relative_path,
						'img_url'		=>	$image_url,
						'img_width'		=>	$width,
						'img_height'	=>	$height,
						'img_attr'		=>	'width="'.$width.'" height="'.$height.'"',
						'img_size'		=>	bytes_to_kb($statinfo['size']),
						'img_bytes'		=>	$statinfo['size'],
						'thumb_url'		=>	absolute_to_url($thumb_filename),
						'thumb_width'	=>	$this->thumb_width,
						'thumb_height'	=>	$this->thumb_height
					);
					
					return true;
				} else {
					unlink($this->working);
					$this->error = 'error uploading';
					return false;
				}
			} else {
				unlink($this->working);
				return false;				
			}
		} else { // Invalid data
			return false;
		}
	}
		 
	
	/**
	 * valid_data
	 * Validate the whole data before do the thing.
	 *
	 * @access	public
	 * @param	string
	 * @return	mixed
	 */
	private function valid_data()
	{
		// Sourced?
		if(!check_value($this->source)) {
			$this->error = 'empty source';
		} else { // it's sourced.. But is valid?
			if($this->is_remote) {
				// URL Check...
				if(!$this->valid_url()) $this->error = 'invalid image url';
			} else {
				// $_FILES Check...
				if(count($this->source)<5 and !is_numeric($this->source['size'])) $this->error = 'invalid source';
			}
		}
		
		// Valid thumb size?
		if(!is_numeric($this->thumb_width) or !is_numeric($this->thumb_height)) {
			$this->error = 'invalid thumb size';
		}
		
		// Setted and valid resize?
		if($this->seted_value($this->resize_width) and !is_numeric($this->resize_width)) {
			$this->error = 'invalid resize size';
		}		
		
		// No errors... Process the thing
		if(!check_value($this->error) and $this->fetch_image()) {
			$this->info = $this->get_info($this->working);
			if(!check_value($this->info)) {
				unlink($this->working);
				$this->error = 'invalid image';
				return false;
			} else {
				// The image is valid... Check for valid MIME
				if(!$this->valid_mime($this->info['mime'])) {
					$this->error = 'invalid mime';
					return false;
				} else {
					$this->mime = $this->info['mime'];
					$statinfo = stat($this->working);
					// The MIME is valid... Check for valid size (bytes)
					if($statinfo['size']>$this->max_size) {
						unlink($this->working);
						$this->error = 'too big';
						return false;
					} else {
						return true; // All valid!
					}
				}
			}
		} else {
			return false; // errors = false
		}
	}
	
	 
	/**
	 * fetch_image
	 * Fetch the current image
	 * 
	 * @access	private
	 * @param	string
	 * @return	string
	 */
	private function fetch_image()
	{
		$destination = $this->img_upload_path.'temp_'.generateRandomString(256);
		
		if($this->is_remote) {
			/**
			 * Upload method: URL
			 */
			$ch = curl_init();
			curl_setopt ($ch, CURLOPT_URL, $this->source);
			curl_setopt ($ch, CURLOPT_RETURNTRANSFER, true);
			curl_setopt ($ch, CURLOPT_HEADER, false);
			curl_setopt ($ch, CURLOPT_AUTOREFERER, true);
			curl_setopt ($ch, CURLOPT_TIMEOUT, 120);
			// We upload the image to the temp directory
			$out = fopen($destination, 'wb');
			curl_setopt($ch, CURLOPT_FILE, $out);
			curl_exec($ch);
			fclose($out);
			// Checkout the errors and image size using the cURL data
			if(!curl_errno($ch)) {
				if(curl_getinfo($ch, CURLINFO_SIZE_DOWNLOAD)>$this->max_size) {
					unlink($destination);
					$this->error = 'too big';
				} else {
					$result = true;
				}
			}
			curl_close($ch);
		} else {
			/**
			 * Upload method: local
			 *
			 * Notice that $_FILES['filename']['type'] returns mime according with the
			 * file extension even if is invalid. This is just a .ext filter and later
			 * the mime will be really checked.
			 */
			if($this->max_size>$this->source['size']) {
				if($this->valid_extension($this->source['name'])) {
					$result = @rename($this->source['tmp_name'], $destination); // For API... RENAME
				} else {
					$this->error = 'invalid extension';
				}
			} else {
				$this->error = 'too big';
			}
		}
		
		if(!$result) {
			if(!check_value($this->error)) $this->error = 'error uploading';
			return false;
		} else {
			$this->working = $destination;
			return true;
		}
	}


	/**
	 * resize_image
	 * Resizes the image (thumbnail and resize feature)
	 * 
	 * @access	private
	 * @param	string
	 * @return	string
	 */
	private function resize($source, $destination, $width, $height='', $thumb=false)
	{
	
		$extension = $this->extension;
		
		switch($extension) {
			case 'gif':
				$src = @imagecreatefromgif($source);
			break;
			case 'png':
				$src = @imagecreatefrompng($source);
			break;
			case 'jpg':
				$src = @imagecreatefromjpeg($source);
			break;
		}
		
		// If the SRC is valid we do the thing
		if($src) {

			$imageSX = imageSX($src);
			$imageSY = imageSY($src);
			
			if($thumb) {
				$source_ratio = $imageSX/$imageSY;
				$destination_ratio = $width/$height;
				
				// Ratio
				if ($destination_ratio>$source_ratio) {
				   $ratio_height = round($width/$source_ratio);
				   $ratio_width = $width;
				} else {
				   $ratio_width = round($height*$source_ratio);
				   $ratio_height = $height;
				}
				
				$target = imagecreatetruecolor($ratio_width, $ratio_height);
				
				$x_center = $ratio_width/2;
				$y_center = $ratio_height/2;
				
			} else {
				$height = round($width * $this->info['1']/$this->info['0']);
				$target = imagecreatetruecolor($width, $height);
			}
			
			// Transparency?
			if ($extension=='gif' or $extension=='png') {
				$this->image_transparent($src, $target);
			} else {
				imagecolortransparent($target, imagecolorallocate($target, 0, 0, 0));
			}	
			imagealphablending($target, false);
			imagesavealpha($target, true);
			
			// Copies SRC to TARGET
			if($thumb) {
				// Thumb... Process the width and height
				imagecopyresampled($target, $src, 0, 0, 0, 0, $ratio_width, $ratio_height, $imageSX, $imageSY);
				$thumb = imagecreatetruecolor($width, $height);
				// Transparent thumb
				// NOTE.. This NEEDS to be done again to create a good looking thumbs.
				if ($extension=='gif' or $extension=='png') {
					$this->image_transparent($target, $thumb);
				} else {
					imagecolortransparent($target, imagecolorallocate($thumb, 0, 0, 0));
				}
				
				imagealphablending($thumb, false);
				imagesavealpha($thumb, true);

				imagecopyresampled($thumb, $target, 0, 0, ($x_center-($width/2)), ($y_center-($height/2)), $width, $height, $width, $height);
				imagedestroy($target); 
				$process = $thumb;
			} else {
				// Normal resize -> just change size
				imagecopyresampled($target, $src, 0, 0, 0, 0, $width, $height, $imageSX, $imageSY);
				$process = $target;
			}
			// Creates the thumb
			switch($extension) {
				case 'gif':
					imagegif($process, $destination);
				break;
				case 'png':
					imagepng($process, $destination);
				break;
				case 'jpg':
					imagejpeg($process, $destination, 90);
				break;
			}	
			imagedestroy($process); 
			imagedestroy($src);	
		} else {
			$this->error = 'invalid source';
		}
	}
	 

	/**
	 * image_transparent
	 * Fetch the transparecy for the current resize process
	 *
	 * @access	private
	 * @param	string
	 * @return	string
	 */
	private function image_transparent($source, $target)
	{
		$transparent_index = imagecolortransparent($source);
		if ($transparent_index >= 0) {
			$transparent_color = imagecolorsforindex($source, $transparent_index);
			$transparent_index = imagecolorallocate($target, $transparent_color['red'], $transparent_color['green'], $transparent_color['blue']);
			imagefill($target, 0, 0, $transparent_index);
			imagecolortransparent($target, $transparent_index);
		} else {
			$color = imagecolorallocatealpha($target, 0, 0, 0, 127);
			imagefill($target, 0, 0, $color);
		}
	}
	

	/**
	 * valid_url
	 * This checks for valid URL
	 *
	 * @access	private
     * @param	string
     * @return	boolean
	 */
	private function valid_url()
	{
		if(is_url($this->source) and $this->valid_extension($this->source)) {
			return true;
		}
	}
	 
	 	
	/**
	 * valid_extension
	 * This checks for valid file extension
	 *
	 * @access	private
     * @param	string
     * @return	boolean
	 */	
	private function valid_extension($file)
	{
		return preg_match('/^.*\.(jpg|jpeg|png|gif|bmp)$/i', $file);
	}


	/**
	 * get_true_extension
	 * Returns the true extension according to the mime
	 *
	 * @access	private
	 * @param	string
	 * @return	string
	 */
	private function get_true_extension($mime)
	{
		if ($mime=='image/gif')	{ $ext = 'gif'; }
		if ($mime=='image/png')	{ $ext = 'png'; }
		if ($mime=='image/pjeg' or $mime=='image/jpeg')		{ $ext = 'jpg'; }
		if ($mime=='image/bmp'	or $mime=='image/x-ms-bmp')	{ $ext = 'bmp'; }
		return $ext;
	}
	
	
	/**
	 * valid_mime
	 * Returns TRUE if the mimetype is suported
	 *
	 * @access	private
	 * @param	string
	 * @return	boolean
	 */
	private function valid_mime($mime)
	{
		return preg_match("@image/(gif|pjpeg|jpeg|png|x-png|bmp|x-ms-bmp)$@", $mime);
	}
	
	
	/**
	 * get_info
	 * Gets the info of the image using getimagesize
	 * 
	 * @access	private
	 * @param	string
	 * @return	string
	 */
	private function get_info($file)
	{
		return @getimagesize($file);
	}
	
	
	/**
	 * randomFile
	 * Generates a new and unused filename
	 *
	 * @autor	fabin dot gnu at gmail dot com
	 * @url		http://www.php.net/manual/es/function.file-exists.php#88607
	 *
	 * @access	private
	 * @param	string
	 * @return	string
	 */
	private function randomFile($folder = '', $extension = '')
	{
		$folder = trim($folder);
		$folder = ($folder == '') ? './' : $folder;
		$filepath = $folder . generateRandomString(5) .'.'. $extension;
		while(file_exists($filepath)) {
			$filepath = $folder . generateRandomString(5) .'.'. $extension;
		}
		return $filepath;
	}

	 
	 /**
	 * seted_value
	 * Checks the seted value of the current $value 
	 *
	 * @access	private
	 * @param	string
	 * @return	boolean
	 */
	 private function seted_value($value)
	 {
		 if(isset($value) and !empty($value)) {
			 return true;
		 }
	 }


}

?>