<?php
/* --------------------------------------------------------------------

  Chevereto
  http://www.chevereto.com/

  @version	2.0.2
  @author	Rodolfo Berríos <http://rodolfoberrios.com/>
  @url		<http://godlikestudio.com>

  Copyright (C) 2008-2011 Rodolfo Berríos. All rights reserved.
  <inbox@rodolfoberrios.com>

  --------------------------------------------------------------------- */

if(!defined('access') or !access) die('This file cannot be directly accessed.');

check_install();


/**
 * version
 * Returns the current script version
 */
function version() {
	return '2.0';
}


/**
 * chevereto_config($config_key)
 * Returns the $config value
 */
function chevereto_config($config_key) {
	global $config;
	return $config[$config_key];
}

/**
 * conditional_config($config_key)
 * Returns true/false according to the state of the config value
 */
function conditional_config($config_key) {
	if(chevereto_config($config_key)==true) {
		return true;
	} else {
		return false;
	}
}

/**
 * check_value
 * Looks for valid value and setted ones.
 */
function check_value($value) {
	if(@count($value)>0 and !@empty($value) and @isset($value)) {
		return true;
	}
}


/**
 * clean_spaces
 * Removes all the spaces on a given string
 */
function clean_spaces($var) {
	return str_replace(' ', '', $var);
}


/**
 * bytes_to_kb
 * Converts bytes to KB
 * If the KB > 1024 we autoconvert to MB
 */
function bytes_to_kb($bytes) {
	$kb = round($bytes*0.0009765625, 1);
	return ($kb>1024) ? kb_to_mb($kb).' MB' : $kb.' KB';	
}


/**
 * kb_to_mb
 * Converts KB to MB
 */
function kb_to_mb($kb) {
	return round(bytes_to_kb($kb), 2);
}


/**
 * mb_to_bytes
 * Converts MB to bytes
 */
function mb_to_bytes($mb) {
	return $mb*1048576;
}


/**
 * absolute_to_relative
 * Converts absolute path to relative path
 */
function absolute_to_relative($filepath) {
	return str_replace(__ROOT_DIR__, __RELATIVE_ROOT__, $filepath);
}


/**
 * absolutepath_to_url
 * Converts absolute path to URL
 */
function absolute_to_url($filepath) {
	return str_replace(__ROOT_DIR__, __BASE_URL__, $filepath);
}


/**
 * chevereto_die
 * This function is an alias of php die() but with html error display :1313:
 */
function chevereto_die($error_msg, $title='', $explain=array()) {
	$what_happend = $explain[0];
	$solution = $explain[1];
	$doctitle = (check_value($title)) ? $title.' error' : 'Error';
	$doctitle .= ' - Chevereto '.version();
	require_once(__PATH_CONTENT__.'system/error.php'); // $error_msg are handled here
	die();
}


/**
 * fetch_url
 * uses cURL to get the contents from a url
 */
function fetch_url($url) {
	$ch = curl_init();
	curl_setopt($ch, CURLOPT_URL, $url);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	curl_setopt($ch, CURLOPT_TIMEOUT, 120);
	$file_get_contents = curl_exec($ch);
	curl_close($ch);
	return $file_get_contents;
}


/**
 * xml_output
 * Outputs the REST_API array to xml
 */
function xml_output($array=array()) {
	ini_set("display_errors", 0);
	header("Last-Modified: ".gmdate("D, d M Y H:i:s")."GMT" );
	header("Cache-Control: no-cache, must-revalidate");
	header("Pragma: no-cache");
	header("Content-Type:text/xml; charset=UTF-8");
	$out = '<?xml version="1.0" encoding="UTF-8"?>'."\n";
	$out .= "<response>\n";
	$out .= "	<status_code>$array[status_code]</status_code>\n";
	$out .= "	<status_txt>$array[status_txt]</status_txt>\n";
	if(count($array['data'])>0) {
		$out .= "	<data>\n";
		foreach($array['data'] as $key => $value) {
			$out .= "		<$key>$value</$key>\n";
		}
		$out .= "	</data>\n";
	}
	$out .= '</response>';
	echo $out;
}


/**
 * json_output
 * Outputs the REST_API array to json
 */
function json_output($array=array(), $callback='') {
	ini_set("display_errors", 0);
	header("Last-Modified: ".gmdate("D, d M Y H:i:s")."GMT" );
	header("Cache-Control: no-cache, must-revalidate");
	header("Pragma: no-cache");
	header("Content-type: application/json; charset=UTF-8");
	
	if (check_value($callback) and preg_match('/\W/', $callback)) {
		header('HTTP/1.1 400 Bad Request');
		$json_fail = array('status_code' => '400', 'status_txt' => 'Bad Request');
		exit(json_encode($json_fail));
	}
	
	print sprintf('%s(%s);', $callback, json_encode($array));
}


/**
 * check_install
 * This checks folders + permissions, .php files and settings.
 */
function check_install(){
	global $config, $install_errors;
	
	$image_folders = array(__PATH_IMAGES__, __PATH_THUMBS__);
	$include_files = array(
		__PATH_INCLUDES__.'chevereto.php',
		__PATH_INCLUDES__.'uploader.php',
		__PATH_INCLUDES__.'shorturl.php',
	);
	$classes_files = array(
		__PATH_CLASSES__.'class.handler.php',
		__PATH_CLASSES__.'class.upload.php',
		__PATH_CLASSES__.'class.imageconvert.php',
		__PATH_CLASSES__.'class.shorturl.php'
	);
	
	// Error friendly messages
	$requirements_error = array(
		'There is a problem regarding server requirements. This means that Chevereto can\'t run because of the following:',
		'Please notice that this issue is because your server setup. If you want to run Chevereto please contact your hosting company or system admin regarding this report.'
	);
	$folder_error = array(
		'There is a problem regarding folders. This means that Chevereto can\'t run because one or more folders required doesn\'t exists.',
		'Please double-check your current setup for the missing files'
	);
	$file_error = array(
		str_replace('folders', 'files', $folder_error[0]),
		str_replace('folders', 'files', $folder_error[1])
	);
	$permission_error = array(
		'There is a problem regarding permissions. This means that Chevereto can\'t upload files because of the following:',
		'Chevereto needs a way to write in this folders. You can do this by doing <a href="http://www.google.com/search?q=chmod+777" target="_blank">chmod 0777</a> on the above folders or use <a href="http://www.suphp.org/" target="_blank">suPHP</a> or <a href="http://httpd.apache.org/docs/current/suexec.html" target="_blank">suEXEC</a> on your server setup.'
	);
	$config_error = array(
		'There is a problem regarding your config setup. This means that Chevereto won\'t run because the config is not valid:',
		'Please double-check your settings in the config.php file.'
	);
	$theme_error = array(
		'There is a problem regarding your current theme. This means that Chevereto won\'t run because <code>'.$config['theme'].'</code> theme is not valid:',
		'Please double-check the theme hierarchy.'
	);
	$theme_data_error = array(
		'There is a problem regarding your current theme data. This means that Chevereto won\'t run because <code>'.$config['theme'].'</code> theme has not valid theme data in <code>style.css</code> header comments:',
		'Please double-check the <code>style.css</code> header comments and refer to the theme documentation page.'
	);
	
	// Check for the server requirements
	if(!check_requirements()) {
		chevereto_die($install_errors, 'System', $requirements_error);
	}
	
	// Check for the image folders
	if(!check_files_folders($image_folders, 'Directory')) {
		chevereto_die($install_errors, 'Folder', $folder_error);
	}
	if(!check_permissions($image_folders)) {
		chevereto_die($install_errors, 'Permissions', $permission_error);
	}
	
	// Check for the include files
	if(!check_files_folders($include_files, 'File')) {
		chevereto_die($install_errors, 'Includes', $file_error);
	}
	
	// Check for the class files
	if(!check_files_folders($classes_files, 'File')) {
		chevereto_die($install_errors, 'Classes', $file_error);
	}
	
	// Check for the current config
	if(!check_config()) {
		chevereto_die($install_errors, 'Config', $config_error);
	}
	
	// Check for the current theme
	if(!check_theme()) {
		chevereto_die($install_errors, 'Theme', $theme_error);
	}
	
	// Check for the current theme data
	if(!check_theme_data()) {
		chevereto_die($install_errors, 'Theme data', $theme_data_error);
	}
}


/**
 * check_requirements
 * This checks upload limit, GD library, Domain and cURL. 
 */
function check_requirements() {
	global $install_errors;
	
	$mod_rw_error = 'Apache <a href="http://httpd.apache.org/docs/2.1/rewrite/rewrite_intro.html" target="_blank">mod_rewrite</a> is not enabled.';
	
	if(version_compare(PHP_VERSION, '5.0.0', '<'))
		$install_errors[] = 'Your server is currently running PHP version '.PHP_VERSION.' and Chevereto needs atleast PHP 5.';
	
	if(!extension_loaded('curl') && !function_exists('curl_init'))
		$install_errors[] = '<a href="http://curl.haxx.se/" target="_blank">cURL</a> is not enabled on your current server setup.';
	
	if (function_exists('apache_get_modules')) {
		if(!in_array('mod_rewrite', apache_get_modules()))
			$install_errors[] = $mod_rw_error;
	} else {
		// As today (Jun 10, 2011) i haven't found a better way to test mod_rewrite in CGI setups. The phpinfo() method is not fail safe either.
	}	
	
	if (!extension_loaded('gd') and !function_exists('gd_info')) {
		$install_errors[] = '<a href="http://www.libgd.org" target="_blank">GD Library</a> is not enabled.';
	} else {
		$imagetype_fail = 'image support is not enabled in your current PHP setup (GD Library).';
		if(!imagetypes() & IMG_PNG)  $install_errors[] = 'PNG '.$imagetype_fail;
		if(!imagetypes() & IMG_GIF)  $install_errors[] = 'GIF '.$imagetype_fail;
		if(!imagetypes() & IMG_JPG)  $install_errors[] = 'JPG '.$imagetype_fail;
		if(!imagetypes() & IMG_WBMP) $install_errors[] = 'BMP '.$imagetype_fail;
	}

	if(count($install_errors)==0) return true;
}


/**
 * check_files_folders
 * This checks for filer or folders existence
 * If the folder doesn't exists the script will attempt to create it.
 */
function check_files_folders($elements, $type) {
	global $install_errors;
	
	foreach ($elements as $element) {
		if($type=='Directory') @mkdir($element);
		if(!file_exists($element)) {
			$install_errors[] = $type.' <code>'.absolute_to_relative($element).'</code> doesn\'t exists';			
		}
	}
	
	if(count($install_errors)==0) return true;
}


/**
 * check_permissions
 * Check the current permission on a array directory
 */
function check_permissions($dirs) {
	global $install_errors;

	foreach ($dirs as $dir) {
		if(!is_writable($dir)) {
			$install_errors[] = 'No write permission in <code>'.absolute_to_relative($dir).'</code> directory.';
		}
	}
		
	if(count($install_errors)==0) return true;
}


/**
 * check_config
 * This checks the script configuration... Like upload limit, thumbs, etc. 
 */
function check_config() {
	global $config, $install_errors;
	
	if(!defined('HTTP_HOST'))
		$install_errors[] = 'Can\'t resolve <code>HTTP_HOST</code>. Please check at the bottom of <code>config.php</code>';
	
	// Upload limit vs php.ini value -> http://php.net/manual/ini.php
	$ini_upload = trim(ini_get('upload_max_filesize'), 'M');
	if(!is_numeric($config['max_size_mb'])) {
		$install_errors[] = 'Invalid numeric value in <code>$config[\'max_size_mb\']</code>';
	} else {
		if($ini_upload<$config['max_size_mb']) {
			$install_errors[] = 'Max. image size ('.$config['max_size_mb'].' MB) is greater than the value in <code>php.ini</code> ('.$ini_upload.' MB)';
		}
	}
	
	if(!is_numeric($config['thumb_width']))
		$install_errors[] = 'Invalid thumb size width in <code>$config[\'thumb_width\']</code>';
	if(!is_numeric($config['thumb_height']))
		$install_errors[] = 'Invalid thumb size height in <code>$config[\'thumb_height\']</code>';
	
	if(!is_numeric($config['min_resize_size']))
		$install_errors[] = 'Invalid minimum resize size in <code>$config[\'min_resize_size\']</code>';
	if(!is_numeric($config['max_resize_size']))
		$install_errors[] = 'Invalid maximum resize size in <code>$config[\'max_resize_size\']</code>';
		
	if($config['short_url_service']=='bitly') {
		$bitly_status = fetch_url('http://api.bit.ly/v3/validate?x_login='.$config['short_url_user'].'&x_apiKey='.$config['short_url_keypass'].'&apiKey='.$config['short_url_keypass'].'&login='.$config['short_url_user'].'&format=json');
		$bitly_json = json_decode($bitly_status);
		
		if($bitly_json->data->valid!==1)
			$install_errors[] = 'The <a href="http://bit.ly/" target="_blank">bit.ly</a> user/api is invalid. bitly server says <code>'.$bitly_json->status_txt.'</code>. Please double check your data.';
	}
	
	if(count($install_errors)==0) return true;
}


/**
 * check_theme
 * This cheks the consistency of the current theme... First the folder and then the files.
 */
function check_theme() {
	global $config, $install_errors;
	
	$theme_files = array(
							'style.css',
							'index.php',
							'header.php',
							'footer.php',
							'uploaded.php',
							'view.php',
							'404.php',
						);
	
	if(!file_exists(__PATH_THEME__)) {
		$install_errors[] = 'Theme directory <code>'.absolute_to_relative(__PATH_THEME__).'</code> doesn\'t exists.';
	} else {
		foreach($theme_files as $theme_file) {
			if(!file_exists(__PATH_THEME__.$theme_file)) {
				$install_errors[] = '<code>'.absolute_to_relative(__PATH_THEME__).$theme_file.'</code> doesn\'t exists.';
			}
		}
	}
	
	if(count($install_errors)==0) return true;
}


/**
 * check_theme_data
 * This cheks the consistency of the current theme data.
 */
function check_theme_data() {
	global $install_errors;
	
	$theme_data = get_theme_data();
	
	if(!check_value($theme_data['Name'])) {
		$install_errors[] = '<code>Theme Name</code> has no value.';
	}
	if(!check_value($theme_data['Chevereto'])) {
		$install_errors[] = 'There is no value on <code>@Chevereto</code> wich indicates the version compatibility.';
	}
	
	if(count($install_errors)==0) return true;
}


/**
 * get_theme_data
 * This gets the data of the current theme.
 * @Inspired from WordPress
 */
function get_theme_data() {
	$theme_file = __PATH_THEME__.'style.css';
	$theme_data = implode('', file($theme_file));
	$theme_data = str_replace ('\r', '\n',$theme_data );
	
	if (preg_match('|Theme Name:(.*)$|mi', $theme_data, $theme_name))
		$name = clean_header_comment($theme_name[1]);

	if (preg_match('|Theme URL:(.*)$|mi', $theme_data, $theme_url))
		$url = clean_header_comment($theme_url[1]);
		
	if (preg_match('|Version:(.*)$|mi', $theme_data, $theme_version))
		$version = clean_header_comment($theme_version[1]);
	
	if (preg_match('|Author:(.*)$|mi', $theme_data, $theme_author))
		$author = clean_header_comment($theme_author[1]);
	
	if (preg_match('|@Chevereto:(.*)$|mi', $theme_data, $theme_chevereto))
		$chevereto = clean_header_comment($theme_chevereto[1]);
		
	return array(
				'Name' => $name,
				'URL' => $url,
				'Version' => $version,
				'Author' => $author,
				'Chevereto' => $chevereto
			);
}


/**
 * check_external_mode
 * When external_mode is setted to false,Chevereto avoids posting files from external sites.
 */
function check_external_mode() {
	global $config;
	
	$http_referer = parse_url($_SERVER['HTTP_REFERER']);

	if (!check_value($http_referer['host'])) {
		$referal = HTTP_HOST;
	} else {
		$referal = (check_value($http_referer['port'])) ? $http_referer['host'].':'.$http_referer['port'] : $http_referer['host'];
	}
	
	if ($referal !== HTTP_HOST) {
		$referal_error = array(
			'You are trying to post files from outside '.$config['site_name'].'. We only allow ussage from <b>'.HTTP_HOST.'</b> domain.',
		);
		$referal_what = array(
			'There is appears to be an invalid access or request to '.$config['doctitle'].'. This means that whathever are you tring to do is not accepted because:',
			'Please respect our ussage guidelines. If you think that this is an error or if you need futher assitance please don\'t hesitate to <a href="/contact/">contact us</a>.'
		);
		
		chevereto_die($referal_error, 'Access', $referal_what);
	}
}

/**
 * clean_header_comment
 * @Inspired from WordPress
 */
function clean_header_comment($str) {
	return trim(preg_replace("/\s*(?:\*\/|\?>).*/", '', $str));
}


/**
 * check_email_address
 * This cheks for real email adress.. Uses by now for the contact page demo.
 */
function check_email_address($email) {
	$isValid = true;
	$atIndex = strrpos($email, "@");
	if (is_bool($atIndex) && !$atIndex) {
		$isValid = false;
	}
	else {
		$domain = substr($email, $atIndex+1);
		$local = substr($email, 0, $atIndex);
		$localLen = strlen($local);
		$domainLen = strlen($domain);
		if ($localLen < 1 || $localLen > 64) {
			// local part length exceeded
			$isValid = false;
		}
		else if ($domainLen < 1 || $domainLen > 255) {
			// domain part length exceeded
			$isValid = false;
		}
		else if ($local[0] == '.' || $local[$localLen-1] == '.') {
			// local part starts or ends with '.'
			$isValid = false;
		}
		else if (preg_match('/\\.\\./', $local)) {
			// local part has two consecutive dots
			$isValid = false;
		}
		else if (!preg_match('/^[A-Za-z0-9\\-\\.]+$/', $domain)) {
			// character not valid in domain part
			$isValid = false;
		}
		else if (preg_match('/\\.\\./', $domain)) {
			// domain part has two consecutive dots
			$isValid = false;
		}
		else if (!preg_match('/^(\\\\.|[A-Za-z0-9!#%&`_=\\/$\'*+?^{}|~.-])+$/',
		str_replace("\\\\","",$local)))
		{
		 // character not valid in local part unless 
		 // local part is quoted
		 if (!preg_match('/^"(\\\\"|[^"])+"$/', str_replace("\\\\","",$local))) {
			$isValid = false;
		 }
		}
		if ($isValid && !(checkdnsrr($domain,"MX") || checkdnsrr($domain,"A"))) {
			// domain not found in DNS
			$isValid = false;
		}
	}
	return $isValid;
}



/**
 * is_url
 * This will tell if the string is an url or not
 */
function is_url($string) {
	// First of all, it looks like an URL
	if(preg_match("/\b(?:(?:https?|ftp):\/\/|www\.)[-a-z0-9+&@#\/%?=~_|!:,.;]*[-a-z0-9+&@#\/%=~_|]/i", $string)) {
		// Then we will tell if returns something...
		$ch = curl_init();
		curl_setopt ($ch, CURLOPT_URL, $string);
		curl_setopt ($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt ($ch, CURLOPT_NOBODY, true);
		curl_setopt ($ch, CURLOPT_FAILONERROR, false);
		curl_setopt ($ch, CURLOPT_AUTOREFERER, true);
		curl_setopt ($ch, CURLOPT_TIMEOUT, 120);
		$result = @curl_exec($ch);
		curl_close($ch);
		if($result !== false) {
			return true;
		}
	}
}

/**
 * generateRandomString
 * Generates the random string used in randomFile
 *
 * @autor	fabin dot gnu at gmail dot com
 * @url		http://www.php.net/manual/es/function.file-exists.php#88607
 */
function generateRandomString($length = 8) {   
	$string = "";
	$possible = "0123456789abcdefghjkmnpqrstvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
	for($i=0;$i < $length;$i++) {
		$char = substr($possible, rand(0, strlen($possible)-1), 1);
		if (!strstr($string, $char)) {
			$string .= $char;
		}
	}
	return $string;
}


/**
 * api_key
 * Returns the API key setted in config.php
 */
function api_key() {
	global $config;
	if(check_value($config['api_key'])) {
		return $config['api_key'];
	}
}


/**
 * api_mode
 * Returns the boolean according to the $ask_mode and API mode setted in config.php
 */
function api_mode($ask_mode) {
	global $config;
	$api_config_mode = trim(strtolower($config['api_mode']));
	
	if($api_config_mode == $ask_mode) {
		return true;
	}
}



?>
