<?php
/* --------------------------------------------------------------------

  Chevereto
  http://www.chevereto.com/

  @version	2.0.2
  @author	Rodolfo Berríos <http://rodolfoberrios.com/>
  @url		<http://godlikestudio.com>

  Copyright (C) 2008-2011 Rodolfo Berríos. All rights reserved.
  <inbox@rodolfoberrios.com>

  --------------------------------------------------------------------- */

if(!defined('access') or !access) die('This file cannot be directly accessed.');

/*
 * INCLUDE TAGS
 ******************************************************************************/
 
/**
 * include_theme_header()
 * Includes the header.php file
 */
function include_theme_header() {
	include(__PATH_THEME__.'header.php');
}
/**
 * include_theme_footer()
 * Includes the footer.php file
 */
function include_theme_footer() {
	include(__PATH_THEME__.'footer.php');
}


/*
 * CONDITIONALS TAGS
 ******************************************************************************/

/**
 * is_config_auto_lang()
 * Returns true if the autolang is enabled
 */
function is_config_auto_lang() {
	return conditional_config('auto_lang');
}

/**
 * is_config_multiupload()
 * Returns true if the multiupload is enabled
 */
function is_config_multiupload() {
	return conditional_config('multiupload');
}

/**
 * is_config_over_resize()
 * Returns true if the over_resize is enabled
 */
function is_config_over_resize() {
	return conditional_config('over_resize');
}

/**
 * is_config_error_reporting()
 * Returns true if the error_reporting is enabled
 */
function is_config_error_reporting() {
	return conditional_config('error_reporting');
}

/**
 * is_config_short_url()
 * Returns true if $config['short_url']==true
 */
function is_config_short_url() {
	return conditional_config('short_url');
}

/**
 * is_user_preference_short_url()
 * Returns true if the user wants to short the urls
 */
function is_user_preference_short_url() {
	if (isset($_COOKIE['doShort'])) {
		return true;
	} else {
		return false;
	}
}

/**
 * is_html_tag
 * returns true if the argument is some kind of html <tag>
 * TODO: Validate actual true HTML tags.
 */
function is_html_tag($tag) {
	if(strlen($tag) == strlen(strip_tags($tag))) {
		return false;
	} else {
		return true;
	}
}


/*
 * SETUP INFO TAGS
 ******************************************************************************/

/**
 * get_base_url
 * Retrives the base url of the script
 */
function get_base_url() {
	return __BASE_URL__;
}
function show_base_url() {
	echo get_base_url();
}

/**
 * get_default_lang
 * Returns the default language used
 */
function get_default_lang() {
	return chevereto_config('lang');
}
function show_default_lang() {
	echo get_default_lang();
}

/**
 * get_max_upload_size
 * Returns the max. imagesize by config
 */
function get_max_upload_size() {
	return chevereto_config('max_size_mb').'MB';
}
function show_max_upload_size() {
	echo get_max_upload_size();
}

/**
 * get_tinyurl_service
 * echoes the current short url service
 */
function get_tinyurl_service() {
	global $config;
	switch(chevereto_config('short_url_service')) {
		case 'tinyurl':
		default:
			$tiny_service = 'TinyURL';
		break;
		case 'google':
			$tiny_service = 'Google';
		break;
		case 'isgd':
			$tiny_service = 'is.gd';
		break;
		case 'bitly':
			$tiny_service = 'bit.ly';
		break;
		case 'custom':
			$tiny_service = check_value($config['custom_short_url_service']) ? $config['custom_short_url_service'] : 'custom service';
		break;
	}
	echo $tiny_service;
}
function show_tinyurl_service() {
	echo get_tinyurl_service();
}


/*
 * THEME INFO TAGS
 ******************************************************************************/

/**
 * get_theme_name
 * Returns the theme name
 */
function get_theme_name() {
	return chevereto_config('theme');
}
function show_theme_name() {
	echo get_theme_name();
}
 
/**
 * get_theme_url
 * Returns the theme url
 */
function get_theme_url() {
	return __URL_THEME__;
}
function show_theme_url() {
	echo __URL_THEME__;
}

/**
 * get_theme_imgdir
 * Get the image dir of the current theme
 */
function get_theme_imgdir() {
	return __URL_THEME__.'theme-img/';
}
function show_theme_imgdir() {
	echo get_theme_imgdir();
}


/*
 * LANGUAGE TAGS
 ******************************************************************************/
 
/**
 * get_lang_used
 * Returns the current used language code
 */
function get_lang_used() {
	return basename(dirname(__LANGUAGE_FILE__));
}
function show_lang_used() {
	echo get_lang_used();
}

/**
 * get_lang_txt
 * Returns the current lang string
 */
function get_lang_txt($lang_txt) {
	global $lang;
	return $lang[$lang_txt];
}
function show_lang_txt($lang_txt) {
	echo get_lang_txt($lang_txt);
}


/*
 * IMAGE TAGS
 ******************************************************************************/
 
/**
 * show_img_filename
 * echoes the filename.ext of the image
 */
function show_img_filename() {
	echo get_img_filename();
}

/**
 * show_img_width
 * echoes the image width
 */
function show_img_width() {
	echo get_img_width();
}

/**
 * show_img_height
 * echoes the image height
 */
function show_img_height() {
	echo get_img_height();
}

/**
 * get_img_dimentions
 * Returns the image width x height
 */
function get_img_dimentions() {
	return get_img_width().' x '.get_img_height().' pixels';
}
function show_img_dimentions() {
	echo get_img_dimentions();
}

/**
 * show_img_html_size
 * echoes the image width x height like html attr
 */
function show_img_html_size() {
	echo get_img_html_size();
}

/**
 * show_img_weight
 * echoes the image weight (filesize)
 */
function show_img_weight() {
	echo get_img_weight();
}

/**
 * show_img_url
 * echoes the img url
 */
function show_img_url() {
	echo get_img_url();
}

/**
 * show_img_viewer()
 * echoes the viewer url
 */
function show_img_viewer() {
	echo get_img_viewer();
}

/**
 * get_short_url
 * Returns the short url
 * TODO: by db!
 */
function get_short_url() {
	global $config;
	require_once(__PATH_INCLUDES__.'shorturl.php');
	return $ShortURL->get_ShortURL(get_img_url());
}
function show_short_url() {
	echo get_short_url();
}

/**
 * get_img_html
 * Returns the img as html
 */
function get_img_html() {
	return htmlentities('<img src="'.get_img_url().'" border="0" />');
}
function show_img_html() {
	echo get_img_html();
}

/**
 * get_img_bbcode
 * Returns the img as bbcode
 */
function get_img_bbcode() {
	return htmlentities('[img]'.get_img_url().'[/img]');
}
function show_img_bbcode() {
	echo get_img_bbcode();
}


/**
 * show_thumb_url
 * echoes the thumb real url
 */
function show_thumb_url() {
	echo get_thumb_url();
}

/**
 * get_thumb_html
 * Returns the thumb + img link as html
 */
function get_thumb_html() {
	return htmlentities('<a href="'.get_img_url().'"><img src="'.get_thumb_url().'" border="0" /></a>');
}
function show_thumb_html() {
	echo get_thumb_html();
}

/**
 * get_thumb_bbcode
 * Returns the thumb + img link as bbcode
 */
function get_thumb_bbcode() {
	return htmlentities('[url="'.get_img_url().'"][img]'.get_thumb_url().'[/img][/url]');
}
function show_thumb_bbcode() {
	echo get_thumb_bbcode();
}


/*
 * AUXILIARY TAGS
 ******************************************************************************/

/**
 * show_doctitle
 * echoes the page doctitle
 */
function show_doctitle() {
	echo get_doctitle();
}

/**
 * show_chevereto_header
 * Get the chevereto header including js and doctitle
 */
function show_chevereto_header() {
	$doctitle = check_value(get_doctitle()) ? get_doctitle().' - ' : '';
	echo '<noscript><meta http-equiv="refresh" content="0;url='.__BASE_URL__.'error-javascript" /></noscript>

<script type="text/javascript" src="'.__URL_SYSTEM_JS__.'jquery.min.js"></script>
<script type="text/javascript" src="'.__URL_SYSTEM_JS__.'jquery.validate.min.js"></script>
<script type="text/javascript" src="'.__URL_SYSTEM_JS__.'jquery.easing.js"></script>
<script type="text/javascript" src="'.__URL_SYSTEM_JS__.'swfobject.js"></script>
<script type="text/javascript" src="'.__URL_SYSTEM_JS__.'jquery.uploadify.v2.1.4_chevereto.js"></script>

<script type="text/javascript" src="'.__URL_SYSTEM_JS__.'peafowl.php"></script>

<link href="'.__URL_THEME__.'style.css" rel="stylesheet" type="text/css" />
<link href="'.__URL_THEME__.'uploadify.css" rel="stylesheet" type="text/css" />

<link href="'.__URL_THEME__.'favicon.ico" rel="shortcut icon" type="image/x-icon" />

<meta name="generator" content="Chevereto '.version().'" />

<title>'.$doctitle.chevereto_config('doctitle').'</title>';
}

/**
 * show_page_links
 * echoes the setted pages
 */
function show_page_links($tag) {
	$html_tag = is_html_tag($tag) ? $tag : '';
	$html_closing_tag = html_closing_tag($html_tag);
	require(__PATH_THEME__.'pages/pages_config.php');
	foreach($pages_config as $key => $value) {
		if($value['live']) echo $html_tag.'<a href="'.__BASE_URL__.$key.'">'.$value['title'].'</a>'.$html_closing_tag;
	}
}

/**
 * html_closing_tag
 * returns the closing </tag> for a given <tag>
 */
function html_closing_tag($html_tag) {
	return str_replace('<', '</', $html_tag);
}
 

?>